;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Falling Icicle
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main code 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "MAIN ",pc
    PHB                       
    PHK                       
    PLB                                 
    JSR FallingSpike        
    PLB                       

print "INIT ",pc
    RTL

Tilemap:
    db $E0,$E2
YDisp:
    db $EF,$FF


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite code JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

FallingSpike:
    JSR SubGFX           ; Draw sprite
    LDA $9D              ;\ Fall if sprites locked..?
    BNE Fall             ;/
    JSL $01802A          ; Update speed
	LDA $C2,x            ;\ Branch to fall code based on state
	BNE Fall             ;/
    JSR SubOffscreen0Bnk3; Process when off screen
    STZ $AA,x            ; Freeze the sprite
    JSR SubHorzPosBnk3   ; Check horizontal distance within the player      
	XBA
    LDA $0F              ;\
	REP #$20
    CLC                  ; |
    ADC #$0040             ; | If player not near enough of sprite, return
    CMP #$0080             ; |
	SEP #$20
    BCS Return           ;/
    INC $C2,x            ; Increase sprite state
    LDA #$40             ;\ Set a timer
    STA $1540,x          ;/
Return:
    RTS

Fall:
    LDA $1540,x          ;\ If timer not done yet, freeze sprite
    BNE Freeze           ;/
    JSL $01A7DC          ; Interact with the player
    LDA $1588,x          ;\ If not touching the ground, return
    AND #$04             ; |
    BEQ Return           ;/
	LDA $E4,x	;\
	STA $9A		; |
	LDA $14E0,x	; |
	STA $9B		; | Set coords (from 02E476 in all.log)
	LDA $D8,x	; |
	STA $98		; |
	LDA $14D4,x	; |
	STA $99		;/
    PHB                  ; Preserve data bank
    LDA #$02             ;\
    PHA                  ; | Set new data bank
    PLB                  ;/
    LDA #$FF             ;\ Randomly replace the shatter with a flashing one
    JSL $028663          ;/
    PLB                  ; Retrieve data bank
    STZ $14C8,x          ; Kill sprite
    BRA Return

Freeze:
    STZ $AA,x            ; Freeze the sprite
    BRA Return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Sub Horizontal Position Bank 3
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SubHorzPosBnk3:     LDY #$00                
                    LDA $94         
                    SEC                       
                    SBC $E4,X       
                    STA $0F                   
                    LDA $95       
                    SBC $14E0,X     
                    BPL Return03B828          
                    INY                       
Return03B828:       RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Sprite Graphics Routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SubGFX:
    JSR GetDrawInfo
    PHX
    LDX #$01
Loop:
    PHX

    LDA $00
    STA $0300,y

    LDA $01
    CLC
    ADC YDisp,x
    STA $0301,y

    LDA Tilemap,x
    STA $0302,y
                        
    LDX $15E9
    LDA $1540,x        
    BEQ .NoShaking
    LSR                       
    LSR                       
    AND #$01                
    CLC                       
    ADC $0300,y
    STA $0300,y         
.NoShaking
    LDA $15F6,x
    ORA $64
    STA $0303,y
    PLX

    INY #4
    DEX
    BPL Loop

    PLX
    LDY #$02
    LDA #$01
    JSL $01B7B3
    RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;       Y = index to sprite OAM ($300)
;       $00 = sprite x position relative to screen boarder
;       $01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1:              db $0C,$1C
SPR_T2:              db $01,$02

GetDrawInfo:
                    STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ .ON_SCREEN_X        ;  |
                    INC $15A0,x             ; /

.ON_SCREEN_X
                    LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.w #$0040            ;  |
                    CMP.w #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE .INVALID           ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ .ON_SCREEN_LOOP     ;  | else, go through loop once
                    INY                     ; / 
.ON_SCREEN_LOOP
                    LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC SPR_T1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ .ON_SCREEN_Y        ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA SPR_T2,y            ;  |
                    STA $186C,x             ;  |
.ON_SCREEN_Y
                    DEY                     ;  |
                    BPL .ON_SCREEN_LOOP     ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

.INVALID
                    PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /
                    
                    
                    
DATA_03B83B:                      db $40,$B0
DATA_03B83D:                      db $01,$FF
DATA_03B83F:                      db $30,$C0,$A0,$80,$A0,$40,$60,$B0
DATA_03B847:                      db $01,$FF,$01,$FF,$01,$00,$01,$FF

SubOffscreen0Bnk3:  STZ $03                     ; / 
                    JSR IsSprOffScreenBnk3      ; \ if sprite is not off screen, return 
                    BEQ RETURNONE               ; / 
                    LDA $5B                     ; \  vertical level 
                    AND #$01                    ;  | 
                    BNE VerticalLevelBnk3       ; / 
                    LDA $D8,X                   ; \ 
                    CLC                         ;  | 
                    ADC #$50                    ;  | if the sprite has gone off the bottom of the level... 
                    LDA $14D4,X                 ;  | 
                    ADC #$00                    ;  | 
                    CMP #$02                    ;  | 
                    BPL OffScrEraseSprBnk3      ; /    ...erase the sprite 
                    LDA $167A,X                 ; \ if "process offscreen" flag is set, return 
                    AND #$04                    ;  | 
                    BNE RETURNONE               ; / 
                    LDA $13      
                    AND #$01                
                    ORA $03                   
                    STA $01                   
                    TAY                       
                    LDA $1A    
                    CLC                       
                    ADC DATA_03B83F,Y       
                    ROL $00                   
                    CMP $E4,X       
                    PHP                       
                    LDA $1B    
                    LSR $00                   
                    ADC DATA_03B847,Y       
                    PLP                       
                    SBC $14E0,X     
                    STA $00                   
                    LSR $01                   
                    BCC CODE_03B8A8           
                    EOR #$80                
                    STA $00        
           
CODE_03B8A8:        LDA $00                   
                    BPL RETURNONE   
       
OffScrEraseSprBnk3: LDA $14C8,X                 ; \ If sprite status < 8, permanently erase sprite 
                    CMP #$08                    ;  | 
                    BCC OffScrKillSprBnk3       ; / 
                    LDY $161A,X                 ; \ Branch if should permanently erase sprite 
                    CPY #$FF                    ;  | 
                    BEQ OffScrKillSprBnk3       ; / 
                    LDA #$00                    ; \ Allow sprite to be reloaded by level loading routine 
                    STA $1938,Y                 ; / 
OffScrKillSprBnk3:  STZ $14C8,X 
            
RETURNONE:          RTS                         ; Return 

VerticalLevelBnk3:  LDA $167A,X                 ; \ If "process offscreen" flag is set, return 
                    AND #$04                    ;  | 
                    BNE RETURNONE               ; / 
                    LDA $13                     ; \ Return every other frame 
                    LSR                         ;  | 
                    BCS RETURNONE               ; / 
                    AND #$01                
                    STA $01                   
                    TAY                       
                    LDA $1C    
                    CLC                       
                    ADC DATA_03B83B,Y       
                    ROL $00                   
                    CMP $D8,X       
                    PHP                       
                    LDA $1D  
                    LSR $00                   
                    ADC DATA_03B83D,Y       
                    PLP                       
                    SBC $14D4,X     
                    STA $00                   
                    LDY $01                   
                    BEQ CODE_03B8F5           
                    EOR #$80                
                    STA $00  
                 
CODE_03B8F5:        LDA $00                   
                    BPL RETURNONE          
                    BMI OffScrEraseSprBnk3 
   
IsSprOffScreenBnk3: LDA $15A0,X                 ; \ If sprite is on screen, A = 0  
                    ORA $186C,X                 ;  | 
                    RTS                         ; / Return 